/* Copyright 2005 - 2007 Spring Layout Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Functions in this file relate to the Spring Layout Datagrid. They provide functionality for
 * row manipulation and manipulation of controls within Datagrid rows.
 * 
 * @author Adam Boas
 */

var removeButtonRegex = /_removeControl/;
var cancelButtonRegex = /_cancelControl/;


/**
 * Adds a new row at the bottom of the dataGrid identified by the gridId provided.
 * @param gridId the id of the dataGrid to remove a row from.
 */
function addRow(gridId)
{
   var table = $(gridId);
	var row = $(gridId + "_extra");
   var rowIndex = table.nextIndex;
	var cloneRow = row.cloneNode(true);
	var newRow = table.tBodies[0].insertRow(table.rows.length - 1);
   //This regex matches the path to the collection which the grid is iterating over and captures
   //whatever else is after that.
	var pathRegex = table.path + '\\[-\\d+\\](.*)';
	pathRegex = new RegExp(pathRegex);
	var replacementPath = table.path + '[' + rowIndex + ']';
	
	cloneRow.style.visibility = 'visible';
   cloneRow.style.display = table.rows[0].style.display;
	cloneRow.id = gridId + "_" + rowIndex;
	table.tBodies[0].replaceChild(cloneRow, newRow);

   var controls = getRowControls(cloneRow, true);
   var originalControls = getRowControls(row, true);
   for(var ii = 0; ii < controls.length; ii++)
   {
      if(controls[ii].nodeName == 'BUTTON')
      {
   		if(removeButtonRegex.test(controls[ii].id) )
   		{
   			controls[ii].id = gridId + '_' + rowIndex + '_removeControl';
            controls[ii].onclick = null;
            addEvent(controls[ii], 'click', function() {removeRow(gridId, rowIndex); });
   		}
         else if(cancelButtonRegex.test(controls[ii].id) )
   		{
   			controls[ii].id = gridId + '_' + rowIndex + '_cancelControl';
            controls[ii].onclick = null;
            addEvent(controls[ii], 'click', function() {replaceRow(gridId, rowIndex); });
   		}
      }
		else 
		{
			controls[ii].id = controls[ii].id.replace(pathRegex, replacementPath + '$1');
			controls[ii].name = controls[ii].name.replace(pathRegex, replacementPath + '$1');
         controls[ii].style.visibility = 'visible';
         if(Element.hasClassName(controls[ii], 'dateField'))
         {
            var icon = controls[ii].nextSibling;
            icon.name = 'calendar_' + controls[ii].id;
            icon.id = 'calendar_' + controls[ii].id;
            removeEvent(icon, 'click', showCalendarForIcon);
            removeEvent(icon, 'mouseover', function(){over(this, 'calIcon');});
            removeEvent(icon, 'mouseout', function(){out(this,'calIcon');});
            icon.onclick = null;
            icon.onmouseover = null;
            icon.onmouseout = null;
         }
         if(originalControls[ii].isMandatory != null)
         {
            controls[ii].isMandatory = originalControls[ii].isMandatory;
            controls[ii].isValid = originalControls[ii].isValid;
            controls[ii].registeredAsDependents = originalControls[ii].registeredAsDependents;
            if(controls[ii].registeredAsDependents != null)
            {
               for(var regIndex = 0; regIndex < controls[ii].registeredAsDependents.length; regIndex++)
               {
                  registerFieldAsMandatoryDependent(controls[ii].registeredAsDependents[regIndex], controls[ii]);
               }
            }
            LAYOUT.validate(controls[ii],true, false, false);
         }
         setDisabled(controls[ii], false);
         if(originalControls[ii].cloned)
         {
            originalControls[ii].cloned.push(controls[ii]);
            controls[ii].prototypeControl = originalControls[ii];
            controls[ii].previouslySelected = controls[ii].options[controls[ii].selectedIndex];
            removeAllOtherSelectsFromThis(controls[ii]);
            addEvent(controls[ii], 'change', manageUniqueSelection);
         }
		}
   }
   var addList = document.forms[table.formName]['dataGrid_addList_' + gridId];
   addList.value += addList.value == '' ? rowIndex : ',' + rowIndex;
	table.nextIndex++;
   
   //remove empty row statement if it exists
   var emptyRow = $(gridId + '_empty');
   if(emptyRow != null)
   {
      emptyRow.style.display = 'none';
      //emptyRow.style.visibility = 'collapse';
   }
}

/**
 * Disables all controls in the row indicated by the index of the dataGrid identified
 * by the gridId provided.
 * @param gridId the id of the dataGrid to remove a row from.
 * @param index the index of the row to remove.
 */
function removeRow(gridId, index)
{
   var table = $(gridId);
   var row = $(gridId + '_' + index);
   var controls = getRowControls(row, true);
   for(var ii = 0; ii < controls.length; ii++)
   {
         
      //If it is a layout control then remove the error class and remove from the invalidField list if it is in it
      if(! removeButtonRegex.test(controls[ii].id)  && !cancelButtonRegex.test(controls[ii].id))
      {
         Element.removeClassName(controls[ii], 'error');
         removeFromInvalidFields(controls[ii]);
         setDisabled(controls[ii], true);
      }
   }
   var removeList = document.forms[table.formName]['dataGrid_removeList_' + gridId];
   removeList.value += removeList.value == '' ? index : ',' + index;
   var removeControl = $(gridId + '_' + index + '_removeControl');
   var cancelControl = $(gridId + '_' + index + '_cancelControl');
   removeControl.style.visibility = 'hidden';
   removeControl.style.position = 'absolute';
   cancelControl.style.position = 'relative';
   cancelControl.style.visibility = 'visible';
   checkAllMandatories();
}

/**
 * Re-enables all controls in the row indicated by the index of the dataGrid identified
 * by the gridId provided.
 * @param gridId the id of the dataGrid to re-enable a row of.
 * @param index the index of the row to re-enable.
 */
function replaceRow(gridId, index)
{
   var table = $(gridId);
   var row = $(gridId + '_' + index);
   var controls = getRowControls(row, true);
   for(var ii = 0; ii < controls.length; ii++)
   {
      setDisabled(controls[ii], false);
      if(!controls[ii].nodeName == 'BUTTON')
         LAYOUT.validate(controls[ii],true, false, false);
   }
   var removeList = document.forms[table.formName]['dataGrid_removeList_' + gridId];
   var regCommaAfter = new RegExp(index +',');
   var regCommaBefore = new RegExp(',' + index);
   if(regCommaAfter.test(removeList.value))
      removeList.value = removeList.value.replace(regCommaAfter, '');
   else if(regCommaBefore.test(removeList.value))
      removeList.value = removeList.value.replace(regCommaBefore, '');
   else
      removeList.value = removeList.value.replace(index, '');
   var removeControl = $(gridId + '_' + index + '_removeControl');
   var cancelControl = $(gridId + '_' + index + '_cancelControl');
   cancelControl.style.visibility = 'hidden';
   cancelControl.style.position = 'absolute';
   removeControl.style.position = 'relative';
   removeControl.style.visibility = 'visible';
}

/**
 * @param row the dataGrid row to retreive the controls of.
 * @param swapModes indicates whether the cells of this row should be forced to swap into edit mode
 * when their controls have been retrieved.
 * @return Array of controls which were in the row provided.
 */
function getRowControls(row, swapModes)
{
   var controls = new Array();
   var cells = row.cells;
   for(var ii = 0; ii < cells.length; ii++)
   {
		var cell =cells[ii];
		Element.cleanWhitespace(cell);
      
      if(typeof cell != 'undefined' && cell.childNodes.length > 0)
      {
         var firstChild = cell.childNodes[0];
         var control = desendToControl(firstChild);
         if(control != null)
         {
            controls.push(control);
            if(removeButtonRegex.test(control.id))
               controls.push(control.nextSibling);
         }
         //If the Cell child is a div we are in multi-mode and we need to push the controls into edit
         if(firstChild.nodeName == 'DIV' && typeof swapModes != 'undefined' && swapModes) 
         {
            swapToEditMode(null, control.id, 'noBlur');
         }
      }
   }
   return controls;
}

/**
 * Function to provide a recursive, depth first search of a node and return with the first control 
 * found.
 *
 * @param {Object} node - the DOM node to perform depth first search on.
 */
function desendToControl(node)
{
   var name = node.nodeName;
   if(name == 'SELECT' || name == 'INPUT' || name == 'BUTTON')
      return node;
      
   var result;
   if(node.childNodes.length > 0)
   {
      result = desendToControl(node.childNodes[0]);
      if(result != null)
         return result;
   }
   while(result == null && node.nextSibling != null)
   {
      result = desendToControl(node.nextSibling);
   }
   return result;
}

/**
* This function calculates the current index of the datagrid based on the Button control passed in.
* The function climbs up to the <tr> and extracts the number from the id of the row
* As an example <tr id="parcelGrid_6"> will return 6
*
* @param button Object firing event within datagrid
*
*/
function calculateCurrentIndex(button)
{
   //Ascend until we find the next nearest row. This allows for other HTML tags in the table cell.
	var row = button.parentNode;
   while(row.nodeName != 'TR')
   {
      row = row.parentNode;
   }
   
	var re = new RegExp("[0-9]+$"); //Index may be multi-digit and must be at the end of the id
   var m = re.exec(row.id);
	return m[0];
}

/**
 * Function to load a referencde to select controls in a datagrid which need to have unique selection
 * maintained across all replicated rows in the grid.
 *
 * @param {Object} gridId - the id of the datagrid which contains the select control.
 * @param {Object} selectIds - a comma separated list of Ids of select controls which are to have
 * unique selection maintained across all rows in the grid.
 */
function loadUniqueSelectOptions(gridId, selectIds)
{
   var table = $(gridId);
   table.uniqueSelectControls = new Array();
   
   //Split id list and load select controls into an Array accessible to the grid.
   var selectIds = selectIds.split(',');
   for(var ii = 0; ii < selectIds.length; ii++)
   {
      // selectPrototype is the select control which will be cloned each time a new row is added
      var selectPrototype = document.forms[table.formName][selectIds[ii]];
      if(selectPrototype && selectPrototype.nodeName == 'SELECT')
      {
         table.uniqueSelectControls.push(selectIds[ii]);
         //Keep a reference to all clones which already exist of the select prototype, these are
         //needed for quick access to their option lists.
         selectPrototype.cloned = new Array();
         var allSelects = selectPrototype.cloned;
         var indexRegex = /\[-1\]/;
         for(var controlIndex = table.nextIndex - 1; controlIndex >= 0; controlIndex--)
         {
            var control = document.forms[table.formName][selectIds[ii].replace(indexRegex, '[' + controlIndex + ']')];
            allSelects.push(control);
            control.prototypeControl = selectPrototype;
            control.previouslySelected = control.options[control.selectedIndex];
            addEvent(control, 'change', manageUniqueSelection);
         }
         //If there are already objects in the grid, make sure that other selects don't have their
         //selected value in their options.
         for(var controlIndex = allSelects.length - 1; controlIndex >= 0; controlIndex--)
         {
            var current = allSelects[controlIndex];
            removeFromAllOtherSelects(current, current.options[current.selectedIndex]);
         }
      }
      else
         alert('You have requested uniquness to be maintained on a non-select control: ' + selectIds[ii]);
   }
}

/**
 * Performs additional work that a control in a group being managed for uniqueness needs to be done
 * when it is disabled.
 * @param {Object} selectSource
 */
function manageDisable(selectSource)
{
   replaceOptionInOtherSelects(selectSource, selectSource.options[selectSource.selectedIndex]);
   selectSource.selectedIndex = 0;
}

/**
 * Places the option provided in all other select controls matching the path in the select control 
 * provided.
 * @param {Object} selectSource to exclude from the adding.
 * @param {Object} option to add to all other select controls in the group.
 */
function replaceOptionInOtherSelects(selectSource, option)
{
   var allSelects = selectSource.prototypeControl.cloned;
   if(option.value != '')
   {
      for(var ii = allSelects.length - 1; ii >= 0 ; ii--)
      {
         var control = allSelects[ii];
         if(control.id != selectSource.id)
         {
            var optionCopy = document.createElement("option");
            optionCopy.appendChild(document.createTextNode(option.text));
            optionCopy.setAttribute('value', option.value);
            control.appendChild(optionCopy);
         }
      }
   }
}

/**
 * Removes the option from all other select controls matching the path in the select control provided
 * but leaves the option in the provided option list.
 * @param {Object} selectSource to exclude from removal.
 * @param {Object} option to remove from all other select controls.
 */
function removeFromAllOtherSelects(selectSource, option)
{
   var allSelects = selectSource.prototypeControl.cloned;
   if(option.value != '')
   {
      for(var ii = allSelects.length - 1; ii >= 0 ; ii--)
      {
         if(allSelects[ii].id != selectSource.id)
         {
            removeOptionFromSelect(allSelects[ii], option.value);
         }
      }
   }
}

/**
 * Removes options selected in all other select controls in this controls group from its option list.
 * @param {Object} selectSource to remove options from.
 */
function removeAllOtherSelectsFromThis(selectSource)
{
   var allSelects = selectSource.prototypeControl.cloned;
   
   for(var ii = allSelects.length - 1; ii >= 0 ; ii--)
   {
      if(allSelects[ii].id != selectSource.id)
      {
         removeOptionFromSelect(selectSource, allSelects[ii].value);
      }
   }
}

/**
 * Wrapper funtion to manage maintaining unique selections across a datagrid for a select control.
 * @param {Object} selectSource to maintain uniqueness for.
 */
function manageUniqueSelection(selectSource)
{
   replaceOptionInOtherSelects(this, this.previouslySelected);
   this.previouslySelected = this.options[this.selectedIndex];
   removeFromAllOtherSelects(this, this.previouslySelected);
}

/**
 * Function to remove an option from a select control.
 * @param {Object} selectControl to remove the option from.
 * @param {Object} value of the option to remove.
 */
function removeOptionFromSelect(selectControl, value)
{
   for(var ii = selectControl.options.length - 1; ii >= 0; ii--)
   {
      if(selectControl.options[ii].value == value)
      {
         selectControl.remove(ii);
         return;
      }
   }
}